#!/bin/sh
trap "" HUP INT QUIT TERM ABRT ALRM EXIT KILL STOP
source /etc/vdr.d/conf/gen2vdr.cfg
source /etc/vdr.d/conf/vdr
#set -x
[ -f $0.conf ] && source $0.conf

# processes which are stopping a shutdown
PROC_STOP_SHUTDOWN="${PROC_STOP_SHUTDOWN:=make gcc cc1 mencoder replex yac_exec.sh lame vdrsync.pl vdrburn.sh}"

SVDRPCMD="/usr/bin/svdrpsend.pl"
NVRAMCMD="/usr/bin/nvram-wakeup"
SETTIMERCMD="/_config/bin/set_timer.sh"
NVRAMCFG="/_config/install/nvram-wakeup.conf"
PROC_ALARM="/proc/acpi/alarm"
WAKEUP_FILE="/_config/status/Wakeup"
MANSTART_FILE="/_config/status/ManualStart"
# Minimaler Abstand zu naechsten Timer in Minuten
WAKEUP_DIFF_MINIMUM=15
NEXT_TIMER=$1
CLOCK=$(source /etc/conf.d/clock; echo ${CLOCK})
DISABLE_STR=0

NT=""
POWEROFF_KERNEL=0

err_exit() {
   if [ ! -f $VDR_FORCE_SHUTDOWN ] ; then
      logger -s "Shutdown abgebrochen <$*>"
      $SVDRPCMD MESG "Shutdown abgebrochen"
      $SVDRPCMD MESG "$*"
      exit
   else
      logger -s "Shutdown trotz <$*>"
   fi
}

set_clock() {
   if [ "$DIRECTISA_RTC" = "1" ] ; then
      DIR_ISA="--directisa"
   else
      DIR_ISA="" 
   fi     
   if [ "$CLOCK" = "UTC" ] ; then
      cp="--utc"
   else
      cp="--localtime"
   fi            
   hwclock $DIR_ISA --systohc $cp --noadjfile
   rm -rf /etc/adjtime > /dev/null 2>&1
}

set_nvram_wakeup() {
   logger -s "Next nvram alarm at: $(date -d "1970-01-01 UTC $1 seconds" +"%Y-%m-%d %R")"
   echo "Using nvram wakeup"
   if [ ! -e /dev/rtc ] ; then
      modprobe rtc
      for i in 1 2 3 4 5 ; do
         [ -e /dev/rtc ] && break
	 sleep 1
      done	 	 
   fi      
      
   if [ "$NVRAM_BOARD" != "" ] ;then
      if [ -f "$NVRAMCFG.$NVRAM_BOARD" ] ;then
         NVRAMCMD="$NVRAMCMD -C $NVRAMCFG.$NVRAM_BOARD"
      else
         NVRAMCMD="$NVRAMCMD -I $NVRAM_BOARD"	    
      fi    
   fi
   [ "$DIRECTISA_NVRAM" = "1" ] && NVRAMCMD="$NVRAMCMD --directisa"

   $NVRAMCMD -ls $1
   [ $? -eq 2 ] && err_exit "nvram-wakeup fehlgeschlagen"
   [ $? -eq 1 ] && POWEROFF_KERNEL=1
}

set_acpi_timer() {
   if [ -f $PROC_ALARM ] && [ $1 -gt 0 ] ; then
      # check if NEXT_TIMER is within next 24 hours
      WAKEUP=$(date -d "1 day" +%s)
      if [ $1 -lt $WAKEUP ] ; then
         WAKEUP=$1
      else
         NEXT_TIMER=$WAKEUP
      fi
      if [ "$CLOCK" = "UTC" ] ; then
         dp="-u"
      else
         dp=""
      fi            
      NEXT_ALARM=$(date $dp -d "1970-01-01 UTC $WAKEUP seconds" +"%Y-%m-%d %R")
      logger -s "Next acpi alarm at: $NEXT_ALARM"
      echo -n "$NEXT_ALARM" > $PROC_ALARM
      sleep 2
      echo -n "$NEXT_ALARM" > $PROC_ALARM
   else
      [ $1 -gt 0 ] && err_exit "ACPI wakeup nicht moeglich"
   fi
}

logger -s "$0 -$1-$2-$3-$4-$5-"
[ -f $WAKEUP_FILE ] && rm -f $WAKEUP_FILE 

# check for forced Shutdown
[ -f /tmp/sandboxpids.tmp ] && err_exit "emerge laeuft"
# check if there are some users logged in
if [ "$IGNORE_USER" = "0" ] ; then
   USERS=$(who | wc -l)
   [ "$USERS" != "0" ] && err_exit "$USERS Benutzer sind angemeldet"
fi
for x in ${PROC_STOP_SHUTDOWN} ; do
   [ "$(pidof -x $x)" != "" ] && err_exit "$x laeuft"
done
# check for freevo
if [ -f /tmp/.vdrdisabled ] ; then
   /_config/bin/switch_console.sh -vdr &
   err_exit "VDR ist nicht aktiv"
fi   

ACT_DATE=$(date +%s)

# Naechsten Timer pruefen
if [ $1 -gt $WAKEUP_RESERVE ] ; then
   NEXT_TIMER=$(($1 - $WAKEUP_RESERVE))
   WAKEUP_DIFF=$((($NEXT_TIMER - $ACT_DATE) / 60))
   [ $WAKEUP_DIFF -lt $WAKEUP_DIFF_MINIMUM ] && err_exit "Naechster Timer in $WAKEUP_DIFF Minuten"
else
   NEXT_TIMER=0
fi

# Ist taeglicher Wakeup aktiviert
if [ "$WAKEUP_DURATION" != "" -a $WAKEUP_DURATION -gt 0 ] ; then
   ACT_HOUR=$(date +%k)
   if [ $WAKEUP_HOUR -gt $ACT_HOUR ] ; then
      NEXT_WAKEUP=$(date -d "$WAKEUP_HOUR:00" +"%s")
   else
      NEXT_WAKEUP=$(date -d "$WAKEUP_HOUR:00 1 day" +"%s")
   fi
   NEXT_WAKEUP=$(($NEXT_WAKEUP - $WAKEUP_RESERVE))
   WAKEUP_DIFF=$((($NEXT_WAKEUP - $ACT_DATE) / 60))
   [ $WAKEUP_DIFF -lt $WAKEUP_DIFF_MINIMUM ] && err_exit "Taeglicher Wakeup in $WAKEUP_DIFF Minuten"
   if [ "$NEXT_TIMER" = "0" ] || [ $NEXT_TIMER -gt $NEXT_WAKEUP ] ; then
      NEXT_TIMER=$NEXT_WAKEUP
      [ "$STR_WU_LIMIT" != "" -a $(($WAKEUP_DIFF / 60)) -lt $STR_WU_LIMIT ] && DISABLE_STR=1
   fi
fi

$SVDRPCMD MESG "System wird heruntergefahren"
# Uhrzeit setzen
set_clock

if [ "$WAKEUP_METHOD" = "NVRAM" ] ; then
   set_nvram_wakeup $NEXT_TIMER
elif [ "$WAKEUP_METHOD" = "SET_TIMER" ] ; then
   $SETTIMERCMD $NEXT_TIMER $(($NEXT_TIMER - $ACT_DATE))
elif [ "$WAKEUP_METHOD" = "ACPI" ] ; then
   set_acpi_timer $NEXT_TIMER
elif [ "$WAKEUP_METHOD" = "WAKEUP_BOARD" ] ; then
   /_config/bin/set_wakeupboard_timer.sh $NEXT_TIMER
else
   logger -s "No wakeup possible - shutting down anyway"
fi

# Check for FilesystemCheck
DOCHECK="0"
if [ "$FSCK_WEEK" != "0" ] ; then
   LCHKF="/etc/lastCheck"
   if [ ! -z $LCHKF ] && [ -f $LCHKF ] ; then
      LCHECKED=$(ls --full-time $LCHKF | cut -f 6,7 -d " ")
      lWeek=$(date -d "$LCHECKED" +"%Y%W")
      actWeek=$(date +"%Y%W")
      if [ $(($actWeek-$lWeek)) -ge $FSCK_WEEK ] ; then
         DOCHECK="1"
      fi
   else   
      DOCHECK="1"
   fi
   [ "$DOCHECK" = "1" ] && touch $LCHKF /forcefsck
fi   

# if SUSPEND To RAM is set and last manual Startup is too long ago then halt
if [ "$SHUTDOWN_METHOD" = "STR" ] && [ "$STR_MAN_LIMIT" != "" ] && [ $STR_MAN_LIMIT -gt 0 ] ; then
   if [ -f $MANSTART_FILE ] ; then
      LMS=$(ls --full-time $MANSTART_FILE | cut -f 6,7 -d " ")
      lDay=$(date -d "$LMS" +%-j)
      aDay=$(date +%-j)
      if [ $aDay -lt $lDay ] ; then
         dDiff=$(($aDay + 365 - $lDay))
      else
         dDiff=$(($aDay - $lDay))
      fi	 
      [ $dDiff -gt $STR_MAN_LIMIT ] && DISABLE_STR=1
   fi
fi

if [ "$WAKEONLAN" = "1" ] ; then
   ethtool -s eth0 wol g
   if (grep PCI0 /proc/acpi/wakeup | grep disabled 2>&1 >> /dev/null) ; then
      echo -n PCI0 > /proc/acpi/wakeup
   fi
fi
   
if [ $1 != 0 ] ; then
   NT=$(date -d "1970-01-01 UTC $NEXT_TIMER seconds" +"%Y%m%d%H%M")
   echo "$1;$3;$4" > $WAKEUP_FILE
   touch -t $NT $WAKEUP_FILE
fi

STOP_VDR=1
if [ "$(/_config/bin/query_mb.sh)" = "SMT7020" ] ; then
   SHUTDOWN_CMD="shutdown -h now"
elif [ "$POWEROFF_KERNEL" = "1" ] || [ "$SHUTDOWN_METHOD" = "PowerOffKernel" ]; then
   lilo -R poweroff
   SHUTDOWN_CMD="shutdown -r now"
else
   if [ "$ACPI" = "1" ] && [ "$SHUTDOWN_METHOD" = "STR" ] && [ "$DOCHECK" = "0" ] && [ "$DISABLE_STR" = "0" ] ; then
      SHUTDOWN_CMD="/_config/bin/hibernate.sh"
      STOP_VDR=0
   elif [ "$SHUTDOWN_METHOD" = "Shutdown" ]; then
      SHUTDOWN_CMD="shutdown -h now"
   else
      SHUTDOWN_CMD="halt"
   fi
fi
$SVDRPCMD MESG "System wird heruntergefahren"
logger -s "Shutting down: <$SHUTDOWN_CMD>"
/_config/bin/activy_lcdtimer.sh
[ "$DOCHECK" = "1" ] && $SVDRPCMD MESG "Filesystem Check beim naechsten Start" && sleep 5
touch /tmp/.shutdown
rm -f $STOPVDR_FILE

if [ "$STOP_VDR" = "1" ] ; then
   /etc/init.d/local stop
   /etc/init.d/gui stop
   /etc/init.d/vdr stop
fi
$SHUTDOWN_CMD
